#!/usr/bin/python

__author__ = "bw142"

## this script will calculate per-window diversity, by using a per-site file and a windows list file
# masked randomly selected (continues) 42.8% singleton

import sys
import bisect
import random

ann_file = sys.argv[1]
bin_file = sys.argv[2]
div_file = sys.argv[3]
div_bin_file = sys.argv[4]

maked_ratio = 0.428

### load annotation files
SNP_dict = {}
infile = open(ann_file, "r")
for line in infile:
    line = line.strip("\n")
    ele = line.split("\t")
    if line[0] != "#":
        ID = "\t".join([ele[0], ele[1], ele[3]])### snp_position + reference allele
        if ele[-1] == "include" and ele[10] == "cds:fold4":### only use fold4 site that was labled as "include"
            if ID not in SNP_dict:
                SNP_dict[ID] = [ele[10], ele[11]]### annotation, Bh
            else:
                print "%s occurred twice" % ID
infile.close()
print "annotation file loaded"

### sorted loci in SNP_dict
SNP_list = SNP_dict.keys()
SNP_list.sort()
print len(SNP_list)

### load in bin file
infile = open(bin_file, "r")## read in window list file
num = 0
bin_dict = {}
for line in infile:
    line = line.strip("\n\r")
    ele = line.split("\t")
    if num >0:##skip haed line
        sca = ele[1]
        start = min(int(ele[2]), int(ele[3]))## start position of a window on scaffold
        end = max(int(ele[2]), int(ele[3]))### end posotion of a window on scaffold
        
        length = (end-start)+1
        
        mask_start = start + random.choice(range(int(length*(1-maked_ratio))))# set the start of masked region
        mask_end = mask_start + int(maked_ratio*length)# set the end of mask region
        
        if sca not in bin_dict:
            bin_dict[sca] = [[start, end, mask_start, mask_end, line, 0, 0, 0, 0, 0]]## the last three position are reserved for pi, theta, S and number of sites, S before masking
        else:
            bin_dict[sca].append([start, end, mask_start, mask_end, line, 0, 0, 0, 0, 0])
    num +=1
infile.close()

## load in div file
infile = open(div_file, "r")
num = 0
for line in infile:
    if num > 0:#skip head line
        line = line.strip("\n\r")
        ele = line.split("\t")
        ID = "\t".join(ele[0:3])
        locate = bisect.bisect(SNP_list, ID)
        if locate != 0 and ID == SNP_list[locate -1]:### the loci has a vailed annotation
            if ele[0] in bin_dict:
                for rec in bin_dict[ele[0]]:
                    if int(ele[1]) >= rec[0] and int(ele[1]) <= rec[1]:
                        site_pi = float(ele[5])## Pi
                        site_theta = float(ele[6])## theta
                        site_S = int(ele[7])## S
                        rec[9]+= int(ele[7])## all site before masking
                        if ele[8] == "Yes":#singleton
                            if int(ele[1]) >= rec[2] and int(ele[1]) <= rec[3]:#masked singleton
                                site_pi = 0
                                site_theta = 0
                                site_S = 0
                        rec[5] += site_pi## Pi
                        rec[6] += site_theta## theta
                        rec[7] += site_S## S
                        rec[8] += 1
    num +=1
infile.close()

### all information are keep in bin_dict now, below we will calculate mean pi within each window
### we will only output ranges of each window on chromosome and scaffold, number of site in each window, and mean values of pi, sxy, da in each window. 
outlist = []
for key in bin_dict:
    for rec in  bin_dict[key]:
        n_site = rec[8]###number of site (both variant and non-variant)in each window
        if n_site != 0:
            ave_pi = str(round((rec[5]/n_site),4))## mean pi
            ave_theta = str(round((rec[6]/n_site),4))## mean theta
            S = str(rec[7])
            S1 = str(rec[9])
        else:
            ave_pi = "-9"
            ave_theta = "-9"
            S = "-9"
            S1 = "-9"
        temp = "\t".join([rec[4], str(rec[2]), str(rec[3]), ave_pi, ave_theta, S, str(n_site), S1])
        outlist.append(temp)

outlist.sort(key=lambda x: x.split("\t")[0])##sort windows by chromosome number and central position
outfile = open(div_bin_file, "w")
outfile.write("gene\tsca\tstart\tend\tmask_st\tmask_ed\tPi\tTheta\tS\tsite\tUnmaksedS\n")
outfile.write("\n".join(outlist))
outfile.close()
