# Rsctipt ZeroDepthInterval.R input output
# This code finds the region with continuous 0 depth

# Output format: start position \t end position \t size \n

args <- commandArgs(trailingOnly = TRUE)
input.file <- as.character(args[1])
output.file <- as.character(args[2])

input.cnxn <- file(input.file, open="r")

# Deal with the first line, as the "previous line" of the second line
first.line <- readLines(input.cnxn,n=1,warn=F)
previous.line.vec <- unlist(strsplit(first.line, '\t', fixed=T))
the.chr <- previous.line.vec[1]
previous.pos <- previous.line.vec[2]
previous.depth <- previous.line.vec[3]
if (previous.depth != 0) {  # Change depth to 1 even if it's 2, 3, etc
  previous.depth <- 1
}

vector.start <- c() # a vector recording the start positions
vector.end <- c() # a vector recording the end positions

if (previous.depth == 0) {  # If the first position of this chr is zero, this position won't be recorded by the code below. Put it in.
  vector.start <- c(vector.start,previous.pos)
}

# Same as perl, each time I call the input file connection, the element is used up
while( length(this.line <- readLines(input.cnxn, n=1)) > 0 ){
  this.line.vec <- unlist(strsplit(this.line, '\t', fixed=T))
  this.pos <- this.line.vec[2]
  this.depth <- this.line.vec[3]
  if (this.depth != 0) {  # Change depth to 1 even if it's 2, 3, etc
    this.depth <- 1
  }
  # Then it's simple
  # Do nothing if the previous depth is the same as this current depth
  # If depth different and this depth is 0, that means the current position is the start of an interval
  # If depth different and this depth is 1, that means the previous position is the end of an interval
  if (previous.depth != this.depth) {
    if (this.depth == 0) {
      vector.start <- c(vector.start,this.pos)
    } else if (this.depth != 0) {
      vector.end <- c(vector.end,previous.pos)
    }
  }
  # Then prepare to go to the next position. For the next position, the current position becomes the "previous"
  previous.pos <- this.pos
  previous.depth <- this.depth
}

if (previous.depth == 0) {  # If the last position of this chr is zero, this position won't be recorded by the code above. Put it in.
  vector.end <- c(vector.end,this.pos)
}

close (input.cnxn)
rm(input.cnxn)

size <- as.integer(as.character(vector.end)) - as.integer(as.character(vector.start)) + 1
output.matrix <- cbind(vector.start,vector.end,size)

write.table(output.matrix,file=output.file,quote=F,col.names=F,row.names=F,sep="\t")
